@@ -1,7 +1,7 @@
 ## Bayesian CRM - extending original code of J. Jack Lee and Nan Chen, Department of Biostatistics, the University of Texas M. D. Anderson Cancer Center
 ## Now using exact inference, rjags, R2WinBUGS or BRugs 
 
-### MJS 10/09/13
+### MJS 23/09/15
 
 if(getRversion() >= "2.15.1") globalVariables(c("N1","pow","d","alpha","p2","logit<-","log.alpha","inverse","patient","toxicity","est","target.tox","q2.5","q97.5","q50","q25","q75","ndose","tox.cutpoints","xmin","ymin","xmax","ymax","Loss","Outcome","traj","Statistic","..density..","Toxicity","weight","Method","..count..","rec","obs","count","n"))
 
@@ -51,13 +51,14 @@ if(getRversion() >= "2.15.1") globalVariables(c("N1","pow","d","alpha","p2","log
 #    bugs.directory --> directory that contains the WinBUGS executable if R2WinBUGS is being used, defaults to "C:/Program Files/WinBUGS14/"
 #	plot	     --> Should dose response curve be plotted after each cohort is entered? Defaults to FALSE
 # 	file	     --> name of the file where the dose-response plots are stored, in a pdf format
+#   seed		--> (Optional) Initial seed value (integer).
 #	N --> Final sample size (Deprecated). To be replaced with stop in future versions
 #     tox         --> number of successes (toxicities) (Deprecated)
 #     notox       --> number of failed patients (no-toxicities) (Deprecated)
 # ----------------------------------------------------------------------
 bcrm<-function(stop=list(nmax=NULL,nmtd=NULL,precision=NULL,nmin=NULL),data=NULL,p.tox0=NULL,sdose=NULL,dose=NULL,ff,prior.alpha,cohort=3,target.tox,constrain=TRUE,sdose.calculate="mean",pointest="plugin",tox.cutpoints=NULL,loss=NULL,
 	start=NULL,simulate=FALSE,nsims=1,truep=NULL,threep3=FALSE,
-	method="exact",burnin.itr=2000,production.itr=2000,bugs.directory="c:/Program Files/WinBUGS14/",plot=FALSE,file=NULL,N,tox,notox){
+	method="exact",burnin.itr=2000,production.itr=2000,bugs.directory="c:/Program Files/WinBUGS14/",plot=FALSE,seed=NULL, file=NULL,N,tox,notox){
 
 	# Checks of argument inputs	
 	if(missing(N) & is.null(stop$nmax) & is.null(stop$nmtd) & is.null(stop$precision)) stop("At least one stopping rule must be provided using the stop argument")
@@ -95,6 +96,11 @@ bcrm<-function(stop=list(nmax=NULL,nmtd=NULL,precision=NULL,nmin=NULL),data=NULL
 
      if(ff=="logit2" & (length(prior.alpha[[2]])<2 | length(prior.alpha[[3]])<2)) stop("second and third components of `prior.alpha' must be vectors of size 2")
 
+	# Set seed if specified
+	if(!is.null(seed)){
+		set.seed(seed)
+	}
+
 	if(missing(sdose)){
 		alpha.prior.plug<-if(prior.alpha[[1]]==1){
 			ifelse(sdose.calculate=="mean",prior.alpha[[2]]*prior.alpha[[3]],median(getprior(prior.alpha, 10000)))
@@ -452,7 +458,6 @@ getprior <- function(prior.alpha, n) {
 # ----------------------------------------------------------------------
 Posterior.rjags <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, production.itr)
 {    
-  if(require(rjags) & require(R2WinBUGS)){
     all.patient <- tox + notox
     datan <-all.patient[all.patient!=0]
     datas <-tox[all.patient!=0]
@@ -502,15 +507,18 @@ Posterior.rjags <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, produc
 	   else stop("Functional form not currently available with specified prior distribution")
     }    
 	path.model<-file.path(tempdir(), "model.file.txt")
-	write.model(model.file,path.model)
-	jagsobj<-jags.model(path.model,data=mydata,n.chains=2,quiet=TRUE)
+	R2WinBUGS::write.model(model.file,path.model)
+	inits.list<-if (ff=="logit2")
+			list(list(log.alpha=c(-3,0),.RNG.seed=sample(1:1e6, size=1),.RNG.name="base::Wichmann-Hill"),list(log.alpha=c(-3,0),.RNG.seed=sample(1:1e6, size=1),.RNG.name="base::Wichmann-Hill"))
+		else list(list(alpha=1,.RNG.seed=sample(1:1e6, size=1),.RNG.name="base::Wichmann-Hill"),list(alpha=1,.RNG.seed=sample(1:1e6, size=1),.RNG.name="base::Wichmann-Hill"))
+	
+	jagsobj<-rjags::jags.model(path.model,data=mydata,n.chains=2,quiet=TRUE,inits=inits.list)
 	update(jagsobj,n.iter=burnin.itr,progress.bar="none")
-	tt<-jags.samples(jagsobj, "alpha", n.iter=production.itr/2,progress.bar="none")
+	tt<-rjags::jags.samples(jagsobj, "alpha", n.iter=production.itr/2,progress.bar="none")
 	if(ff=="logit2"){
 		t<-cbind(c(tt$alpha[1,,]),c(tt$alpha[2,,]))
 	} else {   t<- c(tt$alpha) }
     return(t)
-  }
 }
 
 # ----------------------------------------------------------------------
@@ -530,7 +538,6 @@ Posterior.rjags <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, produc
 # ----------------------------------------------------------------------
 Posterior.BRugs <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, production.itr)
 {    
-  if(require(BRugs) & require(R2WinBUGS)){
     all.patient <- tox + notox
     datan <-all.patient[all.patient!=0]
     datas <-tox[all.patient!=0]
@@ -581,15 +588,15 @@ Posterior.BRugs <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, produc
     }    
 	path.model<-file.path(tempdir(), "model.file.txt")
 	path.data<-file.path(tempdir(), "data.file.txt")
-	write.model(model.file,path.model)
-	bugsData(mydata,path.data)
-	BRugsFit(path.model,path.data, inits=NULL, numChains = 2, parametersToSave="alpha",
-	    	nBurnin = burnin.itr, nIter = production.itr/2, BRugsVerbose = FALSE,DIC=F)
+	R2WinBUGS::write.model(model.file,path.model)
+	BRugs::bugsData(mydata,path.data)
+	BRugsSeed<-sample(1:14,1)
+	BRugs::BRugsFit(path.model,path.data, inits=NULL, numChains = 2, parametersToSave="alpha",
+	    	nBurnin = burnin.itr, nIter = production.itr/2, BRugsVerbose = FALSE,DIC=F,seed=BRugsSeed)
 	if(ff=="logit2"){
-		t<-cbind(samplesSample("alpha[1]"),samplesSample("alpha[2]"))
-	} else {   t<- samplesSample("alpha") }
+		t<-cbind(BRugs::samplesSample("alpha[1]"),BRugs::samplesSample("alpha[2]"))
+	} else {   t<- BRugs::samplesSample("alpha") }
     return(t)
-  }
 }
 
 # ----------------------------------------------------------------------
@@ -610,7 +617,6 @@ Posterior.BRugs <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, produc
 # ----------------------------------------------------------------------
 Posterior.R2WinBUGS <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, production.itr,bugs.directory)
 {    
-  if(require(R2WinBUGS)){
     all.patient <- tox + notox
     datan <-all.patient[all.patient!=0]
     datas <-tox[all.patient!=0]
@@ -661,7 +667,8 @@ Posterior.R2WinBUGS <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, pr
 		TwoPLogisticLogNormal            
 	   else stop("Functional form not currently available with specified prior distribution")
     }    
-    res<-bugs(mydata, inits=NULL, parameters, model.file,n.chains=2,n.thin=1,n.iter=burnin.itr+production.itr/2,n.burnin=burnin.itr,DIC=F,bugs.directory=bugs.directory)  
+	R2WinBUGS.seed<-sample(1:1e6,1)
+    res<-R2WinBUGS::bugs(mydata, inits=NULL, parameters, model.file,n.chains=2,n.thin=1,n.iter=burnin.itr+production.itr/2,n.burnin=burnin.itr,DIC=F,bugs.directory=bugs.directory,bugs.seed=R2WinBUGS.seed)  
     if(is.null(dim(res$summary))){
 	    res$summary<-t(as.matrix(res$summary))
 	}
@@ -675,7 +682,6 @@ Posterior.R2WinBUGS <- function(tox, notox,sdose,ff, prior.alpha, burnin.itr, pr
     }
 	t<-apply(res$sims.array,3,rbind)
     return(t)
-  }
 }
 
 # ----------------------------------------------------------------------
@@ -905,7 +911,7 @@ nextdose<-function(samples.alpha,sdose,ff,target.tox,constrain,pointest,current,
 		est<-apply(loss*probs,2,sum)
 		target<-0
 	} else {
-
+    probs<-NULL
 		if(pointest=="plugin"){
 			mean.alpha<-apply(as.matrix(samples.alpha),2,mean)
 			if(length(mean.alpha)>1) mean.alpha<-t(as.matrix(mean.alpha))
@@ -924,7 +930,7 @@ nextdose<-function(samples.alpha,sdose,ff,target.tox,constrain,pointest,current,
 	ndose<-if(!constrain){which.min(abs(est-target))
 		} else {which.min(abs(est[1:min(current+1,k)]-target))}
 	
-	return(list(ndose=ndose,est=est,mean=mean,sd=sd,quantiles=quantiles,target=target))
+	return(list(ndose=ndose,est=est,mean=mean,sd=sd,quantiles=quantiles,target=target,probs=probs))
 }
 
 
@@ -1389,6 +1395,12 @@ print.bcrm<-function(x,...){
 		colnames(tab3)<-x$dose
 		cat("\n Posterior expected loss at each dose: \n")		
 		print(tab3)
+		tab4<-x$ndose[[length(x$ndose)]]$probs
+		colnames(tab4)<-x$dose
+		rownames(tab4)<-levels(cut(0,breaks=c(0,x$tox.cutpoints,1)))
+		cat("\n Posterior probability of dose being in each toxicity interval")
+		names(dimnames(tab4))<-c("Toxicity intervals","Doses")
+		print(tab4)
 	} else if(x$pointest=="plugin"){
 		tab3<-signif(rbind(x$ndose[[length(x$ndose)]]$est),3)
 		colnames(tab3)<-x$dose
@@ -1493,6 +1505,11 @@ print.threep3<-function(x,tox.cutpoints=NULL,dose=NULL,...){
 	tab2<-signif(rbind(exp.tox,rec.tox),3)
 	rownames(tab2)<-c("Experimentation proportion","Recommendation proportion*")
 	names(dimnames(tab2))<-c("","Probability of DLT")
+	
+	tab3<-signif(rbind(x$n.average,x$dlt.average),3)
+	rownames(tab3)<-c("Average number of patients","Average number of DLTs")
+	colnames(tab3)<-dose
+	names(dimnames(tab3))<-c("","Doses")
 
 	print(tab0)
 	cat("\n")
@@ -1500,6 +1517,8 @@ print.threep3<-function(x,tox.cutpoints=NULL,dose=NULL,...){
 	cat("\n")
 	print(tab2)
 	cat("\n * Amongst those trials that recommend an MTD\n")
+	cat("\n")
+	print(tab3)
 }
 
 #####################################################################
@@ -1522,6 +1541,26 @@ print.threep3<-function(x,tox.cutpoints=NULL,dose=NULL,...){
 #       "truep" - true vector of probabilities
 #
 ####################################################################
+#####################################################################
+#
+# threep3 - Generates all possible 3+3 trial pathways, probabilities
+#            of occurrence, MTD recommendation and sample size
+# AUTHORS: Graham Wheeler, Michael Sweeting
+#
+# ARGUMENTS
+# truep - vector of probabilities for DLT at each dose level
+#              to be investigated
+# start - starting dose level for the design (defaults to 1)
+#    dose		--> optional vector of dose labels (for printing and plotting purposes)
+#
+# VALUES
+#       "prob" - probability of each trial occurring
+#       "ssize" - sample size per trial
+#       "mtd" - final MTD recommendation per trial
+#	   "exp" - experimentation proportions across all possible 3+3 trials
+#       "truep" - true vector of probabilities
+#
+####################################################################
 
 threep3<-function(truep,start=1,dose=NULL){
 	# Check that dose is the same length as truep
@@ -1549,6 +1588,7 @@ threep3<-function(truep,start=1,dose=NULL){
 	pmat[pmat[,"tox 1"]>1 & start==1,"stop"]<-1
 	pmat[pmat[,"tox 1"]>1,"desc"]<-1
 	stopped.pmat<-pmat[pmat$stop==1,-2]
+  all.designs<-stopped.pmat
 	prob<-ssize<-mtd<-dlt.no<-NULL
 	exp<-0
 
@@ -1577,9 +1617,15 @@ threep3<-function(truep,start=1,dose=NULL){
 
 		# Prob. that each dose is experimented on and trial occurs
 		exp<-sapply(1:doses,function(j){sum(3*(stopped.pmat[,grep("d",names(stopped.pmat))]==j)*prob/ssize,na.rm=T)})
+		# Average number of people dosed at each level
+		n.average<-sapply(1:doses,function(j){sum(3*(stopped.pmat[,grep("d",names(stopped.pmat))]==j)*prob,na.rm=T)})
 
 		# Number of subjects who have DLT per trial
 		dlt.no<-apply(stopped.pmat[,grep("tox",names(stopped.pmat))],1,sum,na.rm=T)
+		# Average number of DLTs at each dose
+		dlt.average<-sapply(1:doses,function(j){sum((stopped.pmat[,grep("tox",names(stopped.pmat))]*prob)[stopped.pmat[,grep("d",names(stopped.pmat))]==j],na.rm=T)})
+
+		
 	}
 
 	for(i in 3:mcplus1){
@@ -1611,8 +1657,8 @@ threep3<-function(truep,start=1,dose=NULL){
 
 	 pmat[!is.na(pmat[,dd]),dd][cnt>1]<-NA
  	 pmat[is.na(pmat[,dd]),"stop"]<-1
-  	 stopped.pmat<-pmat[pmat$stop==1,-2]
-
+   stopped.pmat<-pmat[pmat$stop==1,-2]
+   all.designs<-rbind(all.designs,stopped.pmat)
 	 if(dim(stopped.pmat)[1]>0){
 	      # Probabilties of stopped 3+3 trials occuring
 		 dose.mat<-stopped.pmat[,grep("d",names(stopped.pmat))[1:(i-1)]]
@@ -1640,12 +1686,19 @@ threep3<-function(truep,start=1,dose=NULL){
 	
 		# Prob. that each dose is experimented on and trial occurs (summing over all trials)
 		exp<-exp+sapply(1:doses,function(j){sum(3*(stopped.pmat[,grep("d",names(stopped.pmat))]==j)*prob.new/ssize.new,na.rm=T)})
+		# Average number of people dosed at each level
+		n.average<-n.average+sapply(1:doses,function(j){sum(3*(stopped.pmat[,grep("d",names(stopped.pmat))]==j)*prob.new,na.rm=T)})
+
 
 		# Number of subjects who have DLT per trial
 		dlt.no<-c(dlt.no,apply(stopped.pmat[,grep("tox",names(stopped.pmat))],1,sum,na.rm=T))
+		# Average number of DLTs at each dose
+		dlt.average<-dlt.average+sapply(1:doses,function(j){sum((stopped.pmat[,grep("tox",names(stopped.pmat))]*prob.new)[stopped.pmat[,grep("d",names(stopped.pmat))]==j],na.rm=T)})
+
+
 	}
 }
-	obj<-list(prob=prob,ssize=ssize,mtd=mtd,exp=exp,dlt.no=dlt.no,truep=truep,dose=dose)
+	obj<-list(prob=prob,ssize=ssize,mtd=mtd,exp=exp,dlt.no=dlt.no,truep=truep,dose=dose,n.average=n.average,dlt.average=dlt.average,all.designs=all.designs)
 	class(obj)<-"threep3"
 	return(obj)
 }
@@ -1654,6 +1707,7 @@ threep3<-function(truep,start=1,dose=NULL){
 
 
 
+
 #-----------------------------------------------------------------------
 ## HT Gamma
 ## WinBUGS code for a hyperbolic tangent model with Gamma prior on alpha 
